from zope.schema import getFieldsInOrder
from zope.formlib.form import FormField
from zope.formlib.form import FormFields
from zope.formlib.form import _initkw
from zope.interface import implements
from zope.interface.interface import InterfaceClass
from zope.schema.interfaces import IField

from zope.formlib.interfaces import IFormFields
from zope.interface import Attribute

class IFormFieldsets(IFormFields):
    """A grouped collection of form fields (IFormField objects).
    """

    id = Attribute("An id for the fieldset")

    label = Attribute("The label used for the fieldset")

    description = Attribute("An optional description used for the fieldset")

    fieldsets = Attribute("The fieldset definitions")

    def __add__(form_fields):
        """Add two form fieldsets (IFormFieldsets) or add additional fields to
        an existing fieldset.

        Return a new IFormFieldsets which is a nested representation of the two.
        
        In order to fullfill the IFormFields API we make all fields available
        from the base IFormFieldsets.
        """

from zope.formlib import form

from Products.Five.formlib.formbase import EditForm

class FieldsetsEditForm(EditForm):
    """An edit form which supports fieldsets."""

    def setUpWidgets(self, ignore_request=False):
        # First part is copied from zope.formlib.form.EditForm licensed under
        # the ZPL 2.1
        self.adapters = {}
        # In order to support fieldsets, we need to setup the widgets on all
        # the fieldsets as well.
        if self.is_fieldsets():
            self.widgets = None
            for fieldset in self.form_fields.fieldsets:
                fieldset.widgets = form.setUpEditWidgets(
                    fieldset, self.prefix, self.context, self.request,
                    adapters=self.adapters, ignore_request=ignore_request
                    )
                if self.widgets is None:
                    self.widgets = fieldset.widgets
                else:
                    self.widgets += fieldset.widgets
        else:
            self.widgets = form.setUpEditWidgets(
                self.form_fields, self.prefix, self.context, self.request,
                adapters=self.adapters, ignore_request=ignore_request
                )

    def is_fieldsets(self):
        # We need to be able to test for non-fieldsets in templates.
        return IFormFieldsets.providedBy(self.form_fields)

class FormFieldsets(FormFields):

    implements(IFormFieldsets)

    id = ''
    label = None
    description = None
    fieldsets = ()

    def __init__(self, *args, **kw):
        # Most of this code is copied from zope.formlib.form.Formfields
        # licensed under the ZPL 2.1
        keep_readonly, omit_readonly, defaults = _initkw(**kw)

        fields = []
        fieldsets = ()
        for arg in args:
            if isinstance(arg, InterfaceClass):
                for name, field in getFieldsInOrder(arg):
                    fields.append((name, field))
            elif IField.providedBy(arg):
                name = arg.__name__
                if not name:
                    raise ValueError("Field has no name")
                fields.append((name, arg))
            elif isinstance(arg, FormFieldsets):
                fieldsets = fieldsets + (arg, )
                for form_field in arg:
                    fields.append((form_field.__name__, form_field))
            elif isinstance(arg, FormFields):
                for form_field in arg:
                    fields.append((form_field.__name__, form_field))
            elif isinstance(arg, FormField):
                fields.append((arg.__name__, arg))
            else:
                raise TypeError("Unrecognized argument type", arg)

        self.fieldsets = fieldsets

        seq = []
        byname = {}
        for name, field in fields:
            if isinstance(field, FormField):
                form_field = field
            else:
                if field.readonly:
                    if omit_readonly and (name not in keep_readonly):
                        continue
                form_field = FormField(field, **defaults)
                name = form_field.__name__

            if name in byname:
                raise ValueError("Duplicate name", name)
            seq.append(form_field)
            byname[name] = form_field

        self.__FormFields_seq__ = seq
        self.__FormFields_byname__ = byname

    def __add__(self, other):
        if not isinstance(other, FormFieldsets) and \
           not isinstance(other, FormFields):
            return NotImplemented
        return self.__class__(self, other)
